<?php
/*
Plugin Name: WP htaccess Control
Plugin URI: http://dardna.com/wp-htaccess-control
Description: Provides an interface to add custom htaccess rules to the htaccess file generated by wordpress. Also provides customisation of the Author Permalink Base.
Version: 1.4.2
Author: António Andrade
Author URI: http://dardna.com
*/
/*  Copyright 2010  António Andrade  (email : dardna@dardna.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
if (!class_exists("WPhtc")) {
	class WPhtc {
		function WPhtc() { 
			}	
		# Replacing the Wordpress Author Permalink by rewriting its rules if we have a Custom Author Permalink, everytime the rules are flush 
		function wphtc_cap($rules){
			$WPhtc_data=get_option('WPhtc_data');
			if(isset($WPhtc_data['cap']) && $WPhtc_data['cap']!=''){
				$rules=unserialize(preg_replace('!s:(\d+):"(.*?)";!se', "'s:'.strlen('$2').':\"$2\";'",str_replace('author/',$WPhtc_data['cap'].'/',serialize($rules))));
				}
			return $rules;
			}
		# Adding our Custom Author Base to the sitemap generated by Google XML Sitemaps
		# Adapted from sitemap-core.php line 2036
		function set_sm(){
			$WPhtc_data=get_option('WPhtc_data');
			if(class_exists('GoogleSitemapGenerator')&&$WPhtc_data['cap']!=''){
				$generatorObject = &GoogleSitemapGenerator::GetInstance();
				if ($generatorObject != null){
					global $wpdb;
					$sql = "SELECT DISTINCT
							p.ID,
							u.user_nicename,
							MAX(p.post_modified_gmt) AS last_post
						FROM
							{$wpdb->users} u,
							{$wpdb->posts} p
						WHERE
							p.post_author = u.ID
							AND p.post_status = 'publish'
							AND p.post_type = 'post'
							AND p.post_password = ''
							" . (floatval($wp_version) < 2.1?"AND p.post_date_gmt <= '" . gmdate('Y-m-d H:i:59') . "'":"") . "
						GROUP BY
							u.ID,
							u.user_nicename";
					$authors = $wpdb->get_results($sql);	
					if($authors && is_array($authors)) {
						foreach($authors as $author) {
							$url=get_bloginfo('home')."/".$WPhtc_data['cap']."/".$author->user_nicename."/";
							$generatorObject->AddUrl($url,$generatorObject->GetTimestampFromMySql($author->last_post),$generatorObject->GetOption("cf_auth"),$generatorObject->GetOption("pr_auth"));
							}
						}
					}
				}
			}
		# Adding rewrite pairs to the .htaccess generated by Wordpress, everytime the rules are flush
		function wphtc_wp_rewrite_rules($wp_rewrite){
			$WPhtc_data=get_option('WPhtc_data');
			for($i=0;$i<count($WPhtc_data['pats']);$i++){
				if($WPhtc_data['pats'][$i]!=''&&$WPhtc_data['subs'][$i]!=''){
					# Adding each custom rewrite pair to $wp_rewrite->non_wp_rules['pattern']='substitution'
					$wp_rewrite->non_wp_rules[$WPhtc_data['pats'][$i]]=$WPhtc_data['subs'][$i];
					}
				}
			# Keeping a copy of the generated htaccess in our option array for later reviewing
			$WPhtc_data['cur_hta']='# BEGIN Wordpress<br/>'.nl2br($wp_rewrite->mod_rewrite_rules()).'# END Wordpress';
			update_option('WPhtc_data',$WPhtc_data);
			return $wp_rewrite;
			}
		# Adding Custom Rules to htaccess file generated by Wordpress, everytime the rules are flush
		function wphtc_hta_rules($rules){
			$WPhtc_data=get_option('WPhtc_data');
			if($WPhtc_data['hta']){
				$new_rules.="\n# WPhtc: Begin Custom htaccess\n";
				$new_rules.=$WPhtc_data['hta']."\n";
				$new_rules.="# WPhtc: End Custom htaccess\n\n";
				}
			# htaccess suggestions
			if($WPhtc_data['disable_serversignature']){
				$new_rules.="# WPhtC: Disable ServerSignature on generated error pages\n";
				$new_rules.="ServerSignature Off\n\n";
				}
			if($WPhtc_data['admin_email']){
				$new_rules.="# WPhtC: Set admin email\n";
				$new_rules.="SetEnv SERVER_ADMIN ".$WPhtc_data['admin_email']."\n\n";
				}
			if($WPhtc_data['disable_indexes']){
				$new_rules.="# WPhtC: Disable directory browsing\n";
				$new_rules.="Options All -Indexes\n\n";
				}
			if($WPhtc_data['up_limit']){
				$new_rules.="# WPhtC: Limit upload size to ".$WPhtc_data['up_limit']." MB\n";
				$new_rules.="LimitRequestBody ".($WPhtc_data['up_limit']*1024*1024)." \n\n";
				}
			if($WPhtc_data['redirect_500']){
				$new_rules.="# WPhtC: Setting 500 Error page\n";
				$new_rules.="ErrorDocument 500 ".$WPhtc_data['redirect_500']."\n\n";
				}
			if($WPhtc_data['protect_wp_config']){
				$new_rules.="# WPhtC: Protect WP-config.php\n";
				$new_rules.="<files wp-config.php>\n";
				$new_rules.="order allow,deny\n";
				$new_rules.="deny from all\n";
				$new_rules.="</files>\n\n";
				}
			if($WPhtc_data['protect_htaccess']){
				$new_rules.="# WPhtC: Protect .htaccess file\n";
				$new_rules.="<files ~ \"^.*\.([Hh][Tt][Aa])\">\n";
				$new_rules.="order allow,deny\n";
				$new_rules.="deny from all\n";
				$new_rules.="</files>\n\n";
				}
			if($WPhtc_data['disable_hotlink']){
				$new_rules.="# WPhtC: Disable image hotlinking\n";
				$new_rules.="<IfModule mod_rewrite.c>\n";
				$new_rules.="RewriteEngine On\n";
				$new_rules.="RewriteCond %{HTTP_REFERER} !^$\n";
				$new_rules.="RewriteCond %{HTTP_REFERER} !^http://(www\.)?".str_ireplace(array("http://","www."),"",get_bloginfo("url"))."/.*$ [NC]\n";
				$new_rules.="RewriteRule \.(jpg|jpeg|png|gif)$ ".$WPhtc_data['disable_hotlink']." [NC,R,L]\n";
				$new_rules.="</IfModule>\n\n";
				}
			if($WPhtc_data['canon']){
				$url=str_ireplace(array("http://","www."),"",get_bloginfo("url"));
				$domain=explode("/",$url);
				$escaped_domain=str_ireplace(".","\.",$domain[0]);
				if($WPhtc_data['canon']=='www'){
					$new_rules.="# WPhtC: Setting WWW canon\n";
					$new_rules.="<IfModule mod_rewrite.c>\n";
					$new_rules.="RewriteEngine On\n";
					$new_rules.="RewriteCond %{HTTP_HOST} ^".$escaped_domain." [NC] \n";
					$new_rules.="RewriteRule (.*) http://www.".$domain[0]."/$1 [R=301,L]\n";
					$new_rules.="</IfModule>\n\n";
					}
				else if($WPhtc_data['canon']=='simple'){
					$new_rules.="# WPhtC: Setting canon to non-WWW\n";
					$new_rules.="<IfModule mod_rewrite.c>\n";
					$new_rules.="RewriteEngine On\n";
					$new_rules.="RewriteCond %{HTTP_HOST} ^www\.".$escaped_domain." [NC] \n";
					$new_rules.="RewriteRule (.*) http://".$domain[0]."/$1 [R=301,L]\n";
					$new_rules.="</IfModule>\n\n";
					}
				}
					// DEPRECATE LATER (mod_rewrite)
					if($WPhtc_data['hta_mr']!=''){
						$insert="\n\n# WPhtc: Begin Custom htaccess\n".$WPhtc_data['hta_mr']."\n# WPhtc: End Custom htaccess\n";
						$new_rules.=str_replace("RewriteEngine On","RewriteEngine On".$insert,$rules);
						}
			else{
				$new_rules.=$rules;
				}
			return $new_rules;
			}
		# WP-htaccess Control options page interface
		function wphtc_page(){
			include (dirname (__FILE__).'/wp-htaccess-control-ui.php');
			}
		# Adding our options page to the admin menu
		function configure_menu(){
			$page=add_submenu_page("options-general.php","WP htaccess Control", "WP htaControl", 6, __FILE__, array('WPhtc','wphtc_page'));
			add_action('admin_print_scripts-'.$page, array('WPhtc','wphtc_page_script'));
			add_action('admin_print_styles-'.$page, array('WPhtc','wphtc_page_style'));
			}
		# Enqueue Ui Scripts on Plugin page
		function wphtc_page_script(){
			wp_enqueue_script("wpHtacControlJS", WP_PLUGIN_URL . '/wp-htaccess-control/wp-htaccess-control-ui.js');  
			}
		# Enqueue Ui Scripts on Plugin page
		function wphtc_page_style(){
			wp_enqueue_style("wpHtacControlCSS", WP_PLUGIN_URL . '/wp-htaccess-control/wp-htaccess-control-ui.css');  
			}
		# Options Page Actions
		function wphtc_page_action(){
			$action=$_POST['action']?$_POST['action']:$_GET['action'];
			global $echo;
			if(isset($action)){
				$WPhtc_data=get_option('WPhtc_data');
				switch($action){
					# if reseting everything just delete the option array
					case 'reset_rules':
						delete_option('WPhtc_data');
						$echo.="All rules reset.";
						global $wp_rewrite;
						$wp_rewrite->flush_rules();
						break;
					# if updating, save new rules in database and flush rewrite rules
					case 'update':	
						# get Rewrite Pairs
						for($i=0;$i<$_POST['WPhtc_num'];$i++){ 
							if(!empty($_POST['WPhtc_pat'.$i]) && !empty($_POST['WPhtc_sub'.$i])){
								$WPhtc_data['pats'][$i]=$_POST['WPhtc_pat'.$i];								
								$WPhtc_data['subs'][$i]=$_POST['WPhtc_sub'.$i];
								}					
							}
						if(count($WPhtc_data['pats'])>1 && $WPhtc_data['pats'][0]==''){
							unset($WPhtc_data['pats'][0],$WPhtc_data['subs'][0]);
							}
						if(is_array($WPhtc_data['pats'])&&is_array($WPhtc_data['subs'])){
							$WPhtc_data['pats']=array_values($WPhtc_data['pats']);
							$WPhtc_data['subs']=array_values($WPhtc_data['subs']);
							}
						# get Custom Htaccess
						$WPhtc_data['hta']=$_POST['WPhtc_hta'];
											// DEPRECATING 
											//$WPhtc_data['hta_mr']=$_POST['WPhtc_hta_mr'];
											unset($WPhtc_data['hta_mr']);
						# get Custom Author Permalink
						$WPhtc_data['cap']=$_POST['WPhtc_cap'];
						# get htaccess Suggestions
						$WPhtc_data['disable_serversignature']=$_POST['WPhtc_disable_serversignature'];
						$WPhtc_data['disable_indexes']=$_POST['WPhtc_disable_indexes'];
						$WPhtc_data['up_limit']=(is_numeric($_POST['WPhtc_up_limit'])&&$_POST['WPhtc_up_limit']>0)?$_POST['WPhtc_up_limit']:'';
						$WPhtc_data['protect_wp_config']=$_POST['WPhtc_protect_wp_config'];
						$WPhtc_data['protect_htaccess']=$_POST['WPhtc_protect_htaccess'];
						$WPhtc_data['disable_hotlink']=trim($_POST['WPhtc_disable_hotlink']);
						$WPhtc_data['redirect_500']=trim($_POST['WPhtc_redirect_500']);
						$WPhtc_data['canon']=$_POST['WPhtc_canon'];
						$WPhtc_data['admin_email']=trim($_POST['WPhtc_admin_email']);
						
						# Unsetting inclusion of Author pages on Google XML Sitemap options, we'll add those links to the sitemap later with our Custom Author Permalink
						$WPhtc_data['sm_enabled']=$_POST['WPhtc_sm_enabled'];
						if($WPhtc_data['sm_enabled']==true){
							$SMoptions=get_option("sm_options");
							if($SMoptions&&is_array($SMoptions)){
								$SMoptions=get_option("sm_options");
								$SMoptions['sm_in_auth']=0;
								update_option('sm_options',$SMoptions);
								# Try to rebuild Sitemap
								do_action("sm_rebuild");
								}
							}
						# Update WP htaccess Control options
						update_option('WPhtc_data',$WPhtc_data);
						# Flush Rewrite Rules
						global $wp_rewrite;
						$wp_rewrite->flush_rules();
						$echo.="All options updated.";
						break;
					# deleting individual rewrite pair
					case 'delete_pair':
						if(isset($_GET['pair'])){
							# if it's the last pair
							if($_GET['pair']==0 && !isset($WPhtc_data['pats'][1])){
								unset($WPhtc_data['pats'], $WPhtc_data['subs']);
								$echo.="Wordpress Rewrite pairs reset.";
								}
							else{
								unset($WPhtc_data['pats'][$_GET['pair']],$WPhtc_data['subs'][$_GET['pair']]);
								$WPhtc_data['pats']=array_values($WPhtc_data['pats']);
								$WPhtc_data['subs']=array_values($WPhtc_data['subs']);
								$echo.='Wordpress Rewrite pair deleted!';
								}
							update_option('WPhtc_data',$WPhtc_data);
							global $wp_rewrite;
							$wp_rewrite->flush_rules();
							}
						break;
					}
				}
			}
		}
	} 
if (class_exists("WPhtc")) {
	$WPhtc = new WPhtc();
	}
if (isset($WPhtc)) {
	add_filter('mod_rewrite_rules', array($WPhtc,'wphtc_hta_rules'));
	add_filter('generate_rewrite_rules', array($WPhtc,'wphtc_wp_rewrite_rules'));
	add_filter('rewrite_rules_array',array($WPhtc,'wphtc_cap'));
	add_action('admin_menu', array($WPhtc,'configure_menu'));
	add_action('sm_buildmap',array($WPhtc,'set_sm'));
	}
?>